<?php

namespace Drupal\eca_vbo\Plugin\ECA\Event;

use Drupal\eca\Entity\Objects\EcaEvent;
use Drupal\eca\Event\Tag;
use Drupal\eca\Plugin\ECA\Event\EventBase;
use Drupal\eca_vbo\Event\EcaVboEvents;
use Drupal\eca_vbo\Event\VboCustomAccessEvent;
use Drupal\eca_vbo\Event\VboExecuteEvent;

/**
 * Plugin implementation of the ECA Events regarding VBO.
 *
 * @EcaEvent(
 *   id = "vbo",
 *   deriver = "Drupal\eca_vbo\Plugin\ECA\Event\VboEventDeriver"
 * )
 */
class VboEvent extends EventBase {

  /**
   * {@inheritdoc}
   */
  public static function definitions(): array {
    $actions = [];
    $actions['execute'] = [
      'label' => 'VBO: Execute Views bulk operation',
      'event_name' => EcaVboEvents::EXECUTE,
      'event_class' => VboExecuteEvent::class,
      'tags' => Tag::CONTENT,
    ];
    $actions['custom_access'] = [
      'label' => 'VBO: Custom access for Views bulk operation',
      'event_name' => EcaVboEvents::CUSTOM_ACCESS,
      'event_class' => VboCustomAccessEvent::class,
      'tags' => Tag::BEFORE,
    ];
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function fields(): array {
    $fields = [];
    $fields[] = [
      'name' => 'operation_name',
      'label' => 'Operation name',
      'type' => 'String',
      'description' => '',
    ];
    end($fields);
    $field = &$fields[key($fields)];
    if ($this->eventClass() === VboExecuteEvent::class) {
      $field['description'] = 'The operation name identifies this process and will show up in the bulk operations configuration form as selectable action. If you need custom access handling using the operation name, then make sure that the Views configuration is using the <em>ECA bulk operations</em> Views field plugin.';
    }
    elseif ($this->eventClass() === VboCustomAccessEvent::class) {
      $field['description'] = 'Important note: The operation name is only available, when the operation got executed from the <em>ECA bulk operations</em> Views field plugin.';
    }
    $fields[] = [
      'name' => 'view_id',
      'label' => 'Restrict by view ID',
      'type' => 'String',
      'description' => '',
    ];
    $fields[] = [
      'name' => 'display_id',
      'label' => 'Restrict by view display ID',
      'type' => 'String',
      'description' => '',
    ];
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function lazyLoadingWildcard(string $eca_config_id, EcaEvent $ecaEvent): string {
    $configuration = $ecaEvent->getConfiguration();

    $wildcard = '';
    $operation_names = [];
    if (!empty($configuration['operation_name'])) {
      foreach (explode(',', $configuration['operation_name']) as $operation_name) {
        $operation_name = trim($operation_name);
        if ($operation_name !== '') {
          $operation_names[] = $operation_name;
        }
      }
    }
    if ($operation_names) {
      $wildcard .= implode(',', $operation_names);
    }
    else {
      $wildcard .= '*';
    }

    $wildcard .= '::';
    $view_ids = [];
    if (!empty($configuration['view_id'])) {
      foreach (explode(',', $configuration['view_id']) as $view_id) {
        $view_id = trim($view_id);
        if ($view_id !== '') {
          $view_ids[] = $view_id;
        }
      }
    }
    if ($view_ids) {
      $wildcard .= implode(',', $view_ids);
    }
    else {
      $wildcard .= '*';
    }

    $wildcard .= '::';
    $display_ids = [];
    if (!empty($configuration['display_id'])) {
      foreach (explode(',', $configuration['display_id']) as $display_id) {
        $display_id = trim($display_id);
        if ($display_id !== '') {
          $display_ids[] = $display_id;
        }
      }
    }
    if ($display_ids) {
      $wildcard .= implode(',', $display_ids);
    }
    else {
      $wildcard .= '*';
    }

    return $wildcard;
  }

}
